package compositeRefactoring;

import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IField;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IMember;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.IWorkingCopy;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.corext.codemanipulation.CodeGenerationSettings;
import org.eclipse.jdt.internal.corext.refactoring.base.Refactoring;
import org.eclipse.jdt.internal.corext.refactoring.rename.RenameFieldRefactoring;
import org.eclipse.jdt.internal.corext.refactoring.rename.RenameMethodRefactoring;
import org.eclipse.jdt.internal.corext.refactoring.rename.RenameTypeRefactoring;
import org.eclipse.jdt.internal.corext.refactoring.structure.PullUpRefactoring;
import org.eclipse.jdt.internal.ui.refactoring.PullUpInputPage;

import compositeRefactoring.refactoringComposite.*;
import compositeRefactoring.resources.GetJavaResources;

/** RefactoringComponentFactory is a convenience class for creating simple RefactoringComponents.
 * It takes the role of the org.eclipse.jdt.internal.ui.refactoring.* classes which prepare and create 
 * refactorings. It is also possible to create Refactorings without using RefactoringComponentFactory.
 * There is also a static reference for the top-level <code>RefactoringComponent</code> for passing betweeen
 * action classes.
 */
public class RefactoringComponentFactory {
	private static RefactoringComponent component = null;
	
	/** Sets a static reference to a <code>RefactoringComponent</code>.
	 * @param comp the <code>RefactoringComponent</code> to be set.
	 */
	public static void setRefactoringComponent(RefactoringComponent comp) {
		component = comp;
	}
	/** Returns the static reference of a <code>RefactoringComponent</code>.
	 * @return the <code>RefactoringComponent</code> reference.
	 */
	public static RefactoringComponent getRefactoringComponent() {
		return component;
	}
	
	/** This is an example for the composition of Refactorings. It works on a project called <code>Cars</code>,
	 * which is located in the workspace of Eclipse. Here, the composition is accomplished with a list.
	 */
	public static void example_List_Strategy() {
		System.out.println("REFACTORINGCOMPONENTPOOL: Doing List Strategy Example...\n\n");
		RefactoringComponent[] example = prepareExample();
		RefactoringComponent compExample = new RefactoringComposite_ListImpl("CompExample", example);
		setRefactoringComponent(compExample);		
	}
	/** This is an example for the composition of Refactorings. It works on a project called <code>Cars</code>,
	 * which is located in the workspace of Eclipse. Here, the composition is accomplished with an 
	 * iterator strategy.
	 */
	public static void example_Iterator_Strategy() {
		System.out.println("REFACTORINGCOMPONENTPOOL: Doing Iterator Strategy Example...\n\n");
		RefactoringComponent[] example = prepareExample();
		RefactoringComponent compExample = new RefactoringComposite_IteratorImpl("CompExample", example);
		setRefactoringComponent(compExample);		
	}
	private static RefactoringComponent[] prepareExample() {
		GetJavaResources gcjr = new GetJavaResources();
		
		IJavaProject jproj = null;
		IPackageFragment parts = null;
		ICompilationUnit appJava = null, carJava = null;
		ICompilationUnit ferrari_engineJava = null, ferrari_tireJava = null;
		
			
		//fetch the specific Java resources...
		jproj = gcjr.getJavaProject("Cars");
		parts = gcjr.getJavaPackage("Cars", "parts");
		appJava = gcjr.getJavaCompilationUnit("Cars", "App.java");
		carJava = gcjr.getJavaCompilationUnit("Cars", "Car.java");
		ferrari_engineJava = gcjr.getJavaCompilationUnit("Cars", "parts", "Ferrari_Engine.java");
		ferrari_tireJava = gcjr.getJavaCompilationUnit("Cars", "parts", "Ferrari_Tire.java");
		//get working copies...
		IWorkingCopy appWC = null, carWC = null; 
		IWorkingCopy ferrari_engineWC = null, ferrari_tireWC = null;
		try {
			
			appWC = (IWorkingCopy)(appJava.getWorkingCopy());
			carWC = (IWorkingCopy)(carJava.getWorkingCopy());
			ferrari_engineWC = (IWorkingCopy)(ferrari_engineJava.getWorkingCopy());
			ferrari_tireWC = (IWorkingCopy)(ferrari_tireJava.getWorkingCopy());
						
		}
		catch (JavaModelException jme) {
			jme.printStackTrace();
		}
		//get method, field and type:
		IMethod myMethod = null;
		try {
			myMethod = (carWC.findPrimaryType().getMethods())[1];
		} catch (JavaModelException e) {
			System.out.println(e.getMessage());
		}
		IField myField = carWC.findPrimaryType().getField("lf_tire");
		IType myType = carWC.findPrimaryType();
				
		//make a Composite refactoring:
		RefactoringComponent[] example = new RefactoringNode[3];
		example[0] = newRenameFieldRefactoring(myField, "lf_tire_refactored");
		example[1] = newRenameMethodRefactoring(myMethod, "rotate_Tires_refactored");
		example[2] = newRenameTypeRefactoring(myType, "Car_refactored");
		return example;
	}
	/** creates a <code>RefactoringComponent</code> representing a RenameTypeRefactoring.
	 * @param type the type to be renamed.
	 * @param newName the new name of the type.
	 * @return the <code>RefactoringComponent</code> enclosing the RenameTypeRefactoring.
	 */
	public static RefactoringComponent newRenameTypeRefactoring(IType type, String newName) {
		RenameTypeRefactoring renameType = new RenameTypeRefactoring(type);
		renameType.setNewName(newName);
		return new RefactoringNode(renameType);
	}
	/** creates a <code>RefactoringComponent</code> representing a RenameFieldRefactoring.
	 * @param field the field to be renamed.
	 * @param newName the new name of the type.
	 * @return the <code>RefactoringComponent</code> enclosing the RenameFieldRefactoring.
	 */
	public static RefactoringComponent newRenameFieldRefactoring(IField field, String newName) {
		RenameFieldRefactoring renameField = new RenameFieldRefactoring(field, null, null);
		renameField.setNewName(newName);
		return new RefactoringNode(renameField);
	}
	/** creates a <code>RefactoringComponent</code> representing a RenameMethodRefactoring.
	 * @param method the method to be renamed.
	 * @param newName the new name of the type.
	 * @return the <code>RefactoringComponent</code> enclosing the RenameTMethodRefactoring.
	 */
	public static RefactoringComponent newRenameMethodRefactoring(IMethod method, String newName) {
		RenameMethodRefactoring renameMethod = null;
		try {
			renameMethod = RenameMethodRefactoring.createInstance(method);
		}
		catch(JavaModelException jme) {
			jme.getMessage();
			jme.printStackTrace();
		}
		renameMethod.setNewName(newName);
		return new RefactoringNode(renameMethod);
	}
	/** creates a <code>RefactoringComponent</code> representing a PullUpRefactoring for a method.
	 * @param members the methods to be pulled up. The array's first entry will be pulled up. 
	 * All other entries will be deleted (it is assumed that all members are the same methods but in
	 * different classes with the same superclass).
	 * @return the <code>RefactoringComponent</code> enclosing the RenameTypeRefactoring.
	 */
	public static RefactoringComponent newPullUpMethodRefactoring(IMethod[] members) {
		CodeGenerationSettings settings = new CodeGenerationSettings();
		//only one of the methods needs to be pulled up:
		PullUpRefactoring pullUp = new PullUpRefactoring(new IMethod[] {members[0]}, settings);
		//but all methods must be deleted:
		pullUp.setMethodsToDelete(members);
		return new RefactoringNode(pullUp);		
	}
}
	
