package compositeRefactoring.changes;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.corext.refactoring.base.ChangeAbortException;
import org.eclipse.jdt.internal.corext.refactoring.base.ChangeContext;
import org.eclipse.jdt.internal.corext.refactoring.base.IChange;
import org.eclipse.jdt.internal.corext.refactoring.base.RefactoringStatus;
import org.eclipse.jdt.internal.corext.refactoring.changes.DeleteFileChange;


import org.eclipse.jdt.internal.corext.refactoring.base.Change;

import compositeRefactoring.codemanipulation.CreateClass;
import compositeRefactoring.codemanipulation.CreateCompilationUnit;
import compositeRefactoring.codemanipulation.CreatePackageDeclaration;
import compositeRefactoring.codemanipulation.FormatCode;

/** This class represents an IChange for the refactoring CreateAbstractClassRefactoring.
 * It is created when the method CreateAbstractClassRefactoring.createChange(IProgressMonitor pm) is called.
 * An abstract class will be created an formatted. It is saved in a compilation unit which is stored 
 * in a parent Package.
 */
public class CreateAbstractClassChange extends Change implements IChange {
	
	private String fSource;
	private IPackageFragment fPackage;
	private CreateCompilationUnit fCreateCompilationUnit;
	private ICompilationUnit fCU;
	private CreatePackageDeclaration fCreatePackageDeclaration;
	
	/** creates a new CreateAbstractClassChange object.
	 * @param parentElement the Package in which the new class is to be created.
	 * @param className the name of the abstract class.
	 */
	public CreateAbstractClassChange(IPackageFragment parentElement, String className) {
		fSource = "public abstract class " + className + "{\n}";
		fPackage = parentElement;
		fCreateCompilationUnit = new CreateCompilationUnit(parentElement, (className + ".java"), fSource, false);
	}
	/** performs the change. When this method is called, a .java-file is created and stored on the
	 * filesystem. Its content is "public abstract class 'className' {}".
	 * @param context the change context in which the change is performed
	 * @param pm a progress monitor tracking the changes.
	 */
	public void perform(ChangeContext context, IProgressMonitor pm) throws JavaModelException, ChangeAbortException {
		fCreateCompilationUnit.executeOperation();
		fCU = fCreateCompilationUnit.getCompilationUnit();
		fCreatePackageDeclaration = new CreatePackageDeclaration(fPackage.getElementName(), fCU);
		fCreatePackageDeclaration.executeOperation();
	}
	/** gets the undo change. The undo change deletes the created abstract class.
	 * @return the undo change.
	 */
	public IChange getUndoChange() {
		IFile file = null;
		try {
			file = (IFile) fCU.getCorrespondingResource();
		}
		catch(JavaModelException jme) {
			System.out.println(jme.getMessage());
			jme.printStackTrace();
			return null;
		}
		return new DeleteFileChange(file);
	}
	/** gets the name of this change.
	 */
	public String getName() {
		return "CreateAbstractClassChange";
	}
	/** gets the modified language element. In this case, the modified language element is a package.
	 * @return the modified language element.
	 */
	public Object getModifiedLanguageElement() {
		return fPackage;
	}
	/** gets the new created .java-file.
	 * @return the compilation unit representing the .java-file.
	 */
	public ICompilationUnit getNewCompilationUnit() {
		return fCU;
	}

}
