package compositeRefactoring.changes;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.corext.refactoring.base.Change;
import org.eclipse.jdt.internal.corext.refactoring.base.ChangeAbortException;
import org.eclipse.jdt.internal.corext.refactoring.base.ChangeContext;
import org.eclipse.jdt.internal.corext.refactoring.base.IChange;

import compositeRefactoring.codemanipulation.Inherit;

/** This class represents an IChange for the refactoring InheritRefactoring.
 * It is created when the method InheritRefactoring.createChange(IProgressMonitor pm) is called.
 * The superclass of Base is set to Derived. The codetransformation is
 * <center><code>...class base <b>extends Derived</b>...</code></center>
 */

public class InheritChange extends Change implements IChange {
	private ICompilationUnit fBase, fDerived;
	private ICompilationUnit fOriginalBase;
	private Inherit fInherit;
	
	/** creates an InheritChange object. 
	 * @param base the compilation unit to inherit.
	 * @param derived the compilation unit to be inherited from.
	 * @param inherit the Inherit object for core operations.
	 */
	public InheritChange(ICompilationUnit base, ICompilationUnit derived, Inherit inherit) {
			fBase = base;
			fDerived = derived;
			fInherit = inherit;
	}
	/** performs the change. After this method is called, base extends derived.
	 * @param context the change context in which the change is performed
	 * @param pm a progress monitor tracking the changes.
	 */
	public void perform(ChangeContext context, IProgressMonitor pm)
		throws JavaModelException, ChangeAbortException {
		fInherit.executeOperation();
	}
	/** gets the undo change. The undo change changes the superclass of Base to the original one or to null 
	 * if Base inherits from java.lang.Object.
	 * @return the undo change.
	 */
	public IChange getUndoChange() {
		return new UndoInheritChange(fInherit);
	}
	/** gets the name of this change.
	 */
	public String getName() {
		return "InheritChange";
	}
	/** gets the modified language element. In this case, the modified language element is the base class
	 * which inherits from the derived class.
	 * @return the modified language element.
	 */
	public Object getModifiedLanguageElement() {
		return fBase;
	}
	
}
