package compositeRefactoring.codemanipulation;

import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Vector;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jdt.core.ElementChangedEvent;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaElementDelta;
import org.eclipse.jdt.core.IJavaModelStatusConstants;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.jdom.DOMException;
import org.eclipse.jdt.core.jdom.DOMFactory;
import org.eclipse.jdt.core.jdom.IDOMCompilationUnit;
import org.eclipse.jdt.core.jdom.IDOMFactory;
import org.eclipse.jdt.core.jdom.IDOMMethod;
import org.eclipse.jdt.core.jdom.IDOMNode;
import org.eclipse.jdt.core.jdom.IDOMType;
import org.eclipse.jdt.core.search.SearchEngine;
import org.eclipse.jdt.internal.core.JavaElementDelta;
import org.eclipse.jdt.internal.core.JavaModelOperation;
import org.eclipse.jdt.internal.core.JavaModelStatus;
import org.eclipse.jdt.internal.core.hierarchy.TypeHierarchy;
import org.eclipse.jdt.internal.corext.refactoring.base.RefactoringStatus;
import org.eclipse.jdt.internal.ui.JavaPlugin;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.text.IDocument;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.texteditor.IDocumentProvider;

/** This class creates an inheritence dependency between two classes.
 * To determine whether one class may inherit from another, some conditions 
 * can be checked in this class according to Lance Tokuda (Univeristy of Texas at Austin)
 */
public class Inherit extends JavaModelOperation {
	/*base is subclass, derived is superclass*/
	private ICompilationUnit fBase, fDerived;
	private String fOriginalContentsBase;
	private JavaElementDelta fDelta = null;
	private IDocumentProvider prov;
	private FileEditorInput editorInput;
	private IDocument document;
	
	//JDOM:
	private IDOMType fDOMDerivedType, fDOMBaseType;
	private IDOMCompilationUnit fDOMDerivedCu, fDOMBaseCu;
	
	/** creates an Inherit object.
	 * @param base the subclass in the inheritence dependency
	 * @param derived the superclass in the inheritence dependency
	 */
	public Inherit(ICompilationUnit base, ICompilationUnit derived) {
		super(new IJavaElement[] {base}, new IJavaElement[] {base.getParent()}, false);
		fBase = base;
		fDerived = derived;
		//set original contents:
		try {
			fOriginalContentsBase = fBase.getBuffer().getContents();
		}
		catch(JavaModelException jme) {
			jme.printStackTrace();
		}
		fDOMDerivedCu = getIDOMCu(fDerived); //here, fOriginalContentsBase is set!
		fDOMDerivedType = getIDOMType(fDOMDerivedCu); 
		fDOMBaseCu = getIDOMCu(fBase);
		fDOMBaseType = getIDOMType(fDOMBaseCu);
	}
	
	/* (non java doc) gets the JDOM (org.eclipse.jdt.jdom) representation of a compilation unit
	 */
	private IDOMCompilationUnit getIDOMCu(ICompilationUnit cu) {
		IDOMFactory jDOM = new DOMFactory();
		IDOMCompilationUnit	IDOMCu;
		try {
			IDOMCu = jDOM.createCompilationUnit(cu.getBuffer().getContents(), cu.getElementName());
		}
		catch(JavaModelException jme) {
			jme.printStackTrace();
			return null;
		}
		return IDOMCu;
	}
	
	/* (non java doc) gets the JDOM (org.eclipse.jdt.jdom) representation of 
	 * a primary type (the public type) of a compilation unit
	 */
	private IDOMType getIDOMType(IDOMCompilationUnit parent) {
		
		for (Enumeration enum = parent.getChildren(); enum.hasMoreElements() ;) {
			IDOMNode node = (IDOMNode) enum.nextElement();
			if (node.getNodeType() == IDOMNode.TYPE) {
				//must be primary type, cannot be an inner class or type.
				return (IDOMType)node;
			}
		}
		return null;
	}
				
	
	/** checks if Base is superclass of Derived. It also determines if Base and Derived are the same classes.
	 * @return <b>true</b> if Base is no superclass of Derived, <b>false</b> otherwise.
	 */
	public RefactoringStatus checkCycle() {
		RefactoringStatus status = new RefactoringStatus();
		String superclass = fDOMDerivedType.getSuperclass();
		if (superclass == null) {
			return status;
		}
		if (superclass.equals(fDOMBaseType.getName())) {
			status.addError("Base is superclass from Derived!");
		}
		//DEBUG:
		System.out.println("INHERIT: " + fBase.getElementName() + ", " + fDerived.getElementName());
		if (fBase.getElementName().equals(fDerived.getElementName())) {
			status.addError("Base cannot inherit from itself!");
		}
		return status;
	}
	/** checks if Base already has a superclass.
	 * @return <b>true</b> if Base inherits from <code>java.lang.Object</code>, <b>false</b> otherwise.
	 */
	public RefactoringStatus checkBaseSuperclass() {
		RefactoringStatus status = new RefactoringStatus();
		if (fDOMBaseType.getSuperclass() != null) {
			status.addError("Base already has a supreclass");
		}
		return status;
	}
	/** checks if all abstract methods from Derived are implemented in Base.
	 * @return <b>true</b> if all abstract methods are implemented, <b>false</b> otherwise.
	 */
	public RefactoringStatus checkAbstractMethods() {
		RefactoringStatus status = new RefactoringStatus();
		Vector abstractMethods = new Vector();
		//first, get all abstract methods of Derived:
		for (Enumeration enum = fDOMDerivedType.getChildren(); enum.hasMoreElements();) {
			IDOMNode node = (IDOMNode) enum.nextElement();
			if(node.getNodeType() == IDOMNode.METHOD) {
				if (isAbstract((IDOMMethod)node, fDOMDerivedType, fDerived)) {
					abstractMethods.add(node);
				}
			}
		}
		//now, compare signatures of the abstract methods with the base methods.
		int counter = abstractMethods.size();
		for (Enumeration enum = fDOMBaseType.getChildren(); enum.hasMoreElements();) {
			IDOMNode node = (IDOMNode) enum.nextElement();
			if(node.getNodeType() == IDOMNode.METHOD) {
				for (int i = 0; i < abstractMethods.size(); i++) {
					if (isImplementation((IDOMMethod)node, (IDOMMethod)abstractMethods.elementAt(i))) {
						counter--;
					}
				}
			}
		}
		if (counter != 0) {
			status.addError("Not all abstract methods of Derived are implemented in Base.");
		}
		return status;
	}
	
	/* (non java doc)
	 * checks whether the impl method is an implementation of abstr.
	 * That is, abstr is an abstract method and impl is located in a subclass
	 * and implements this method. For this, the signature of the two methods must be the same.
	 */
	private boolean isImplementation(IDOMMethod impl, IDOMMethod abstr) {
		boolean temp = false;
		String[] parameterImpl, parameterAbstr;
		if (!(impl.getName().equals(abstr.getName()))) 
			return false;
		if (!(impl.getReturnType().equals(abstr.getReturnType())))
			return false;
		parameterImpl = impl.getParameterTypes();
		parameterAbstr = abstr.getParameterTypes();
		if (parameterImpl.length != parameterAbstr.length) 
			return false;
		for (int i = 0; i < parameterImpl.length; i++) {
			if (!(parameterImpl[i].equals(parameterAbstr[i]))) {
				return false;
			}
		}
		parameterImpl = impl.getParameterNames();
		parameterAbstr = abstr.getParameterNames();
		for (int i = 0; i < parameterImpl.length; i++) {
			if (!(parameterImpl[i].equals(parameterAbstr[i]))) {
				return false;
			}
		}
		return true;
	}
	
	/* (non java doc) 
	 * checks whether the method is abstract.
	 * The problem is: there is no possibility to access the source of the method
	 * via the JDOM representation. Thus, the original (org.eclipse.jdt.core.IMethod) must be fetched.
	 * This is only possible by calling getJavaElement() which requests the JDOM element's parent.
	 * The parent must be a IJavaElement. 
	 * This is why the parent's IJavaElement is fetched at first.
	 */	
	private boolean isAbstract(IDOMMethod method, IDOMType parent, ICompilationUnit parentParent) {
		IMethod coreMethod = (IMethod) method.getJavaElement(parent.getJavaElement(parentParent));
		try {
			StringTokenizer st = new StringTokenizer(coreMethod.getSource());
			while(st.hasMoreTokens()) {
				String token = st.nextToken();
				if (token.equals("abstract")) {
					return true;
				}
				//stop when the body begins, there is no abstract in the signature.
				if (token.equals("{")) {
					return false;
				}
			}
		}
		catch(JavaModelException jme) {
			jme.printStackTrace();
		}
		return false;
	}		
	
	/** executes the inherit operation.
	 */
	public void executeOperation() throws JavaModelException {
		fDelta = newJavaElementDelta(); //from JavaModelOperation
		if (doInheritChange(new NullProgressMonitor())) {
			fResultElements = new IJavaElement[] {fBase.findPrimaryType()};
			fDelta.changed(fResultElements[0], IJavaElementDelta.F_CONTENT);
			addDelta(fDelta);
		}	
		else {
			throw new JavaModelException(new JavaModelStatus(IJavaModelStatusConstants.DOM_EXCEPTION));
		}
	}
	private boolean doInheritChange(IProgressMonitor pm) {
	 	if (fBase == null || fDerived == null) {
			System.out.println("no inherit operation possible, one of the arguments is null");
			return false;
	 	}
	 	/*look if there is a working copy open in the java editor
		 * this is made because of sync reasons:
		 * if there is a working copy, work on the working copy to 
		 * ensure the changes to appear in the editor as well.
		 */
		try {
			editorInput = new FileEditorInput((IFile)fBase.getUnderlyingResource());
		}
		catch (JavaModelException jme) {
			jme.printStackTrace();
		}		
    	ICompilationUnit workingCopy= JavaUI.getWorkingCopyManager().getWorkingCopy(editorInput);
    	if (workingCopy != null) {
    		/* this means there is an editor open showing the file
    		 * which is to be manipulated. Change the working copy,
    		 * not the original file (compilation unit):
    		 */
    		//do the change on the WORKINGCOPY:
    		workingCopy = changeBase(workingCopy);
    		try {
    			commitChanges(new NullProgressMonitor(), true);
    		}
    		catch(CoreException ce) {
    			ce.printStackTrace();
    			return false;
    		}
    		return true;
    	}
    	else {
    		/* no editor is open, change the original
    		 * file (compilation unit), don't care about
    		 * any working copies:
    		 */
    		 fBase = changeBase(fBase);
    		 try {
    		 	fBase.save(null, false);
    		 }
    		 catch(JavaModelException jme) {
    		 	jme.printStackTrace();
    		 }    	
    		 return true;	 
    	}
	}

	/* (non java doc)
	 * This method sets the superclass of the Base's JDOM representation.
	 * Afterwards, it sets the new contents in the original (org.eclipse.jdt.core.ICompilationUnit))
	 */
	private ICompilationUnit changeBase(ICompilationUnit base) {
		
		IDOMType fDOMBaseTypeClone = (IDOMType)fDOMBaseType.clone();
		fDOMBaseTypeClone.setSuperclass(fDOMDerivedType.getName());
		//remove old type:
		fDOMBaseType.remove();
		//add new type:
		fDOMBaseCu.addChild(fDOMBaseTypeClone);
		String contents = fDOMBaseCu.getContents();
		try {
			base.getBuffer().setContents(contents);
			if (base.isWorkingCopy()) {
				base.reconcile();
			}
		} 
		catch(JavaModelException jme) {
			jme.printStackTrace();
		}
		return base;
	}
	/* (non java doc)
	 * This method saves the changes if any changes were made at a working copy.
	 * The working copy must be in a FileEditorInput object, that is, an open editor window in the workbench.
	 * BUG: for now, if there were actual changes made by this object and this method 
	 * is called with force = false, the changes will not be saved, although it is intended.
	 */
	private void commitChanges(IProgressMonitor pm, boolean force) throws CoreException {
		// get the IDocumentProvider. You can save documents only via this provider.
		prov = JavaPlugin.getDefault().getCompilationUnitDocumentProvider();
		if (prov==null) {
			throw new JavaModelException(new JavaModelStatus(IJavaModelStatusConstants.CORE_EXCEPTION));
		}
		try {
			prov.connect(editorInput);
		}
		catch(CoreException ce) {
			ce.printStackTrace();
		}
		document = prov.getDocument(editorInput);
		if (document == null) {
			throw new JavaModelException(new JavaModelStatus(IJavaModelStatusConstants.CORE_EXCEPTION));
		}
		boolean save= force || prov.mustSaveDocument(editorInput);
		if(save) {
			IWorkspaceRunnable action= new IWorkspaceRunnable() {
				public void run(IProgressMonitor pm) throws CoreException {
					prov.aboutToChange(editorInput);
					prov.saveDocument(pm, editorInput, document, true);
				}
			};
			try {
				ResourcesPlugin.getWorkspace().run(action, pm);
			} finally {
				prov.changed(editorInput);
			}
		}
		
		
	}
	
	/** executes the undo operation. It can only be executed after 
	 * running the executeOperation() method on this object. Otherwise it would not make much sense.
	 */
	
	public void executeUndoOperation() throws JavaModelException {
		fDelta = newJavaElementDelta(); //from JavaModelOperation
		if (doUndoChange()) {
			fResultElements = new IJavaElement[] {fBase.findPrimaryType()};
			fDelta.changed(fResultElements[0], IJavaElementDelta.F_CONTENT);
			addDelta(fDelta);
		}
		else {
			throw new JavaModelException(new JavaModelStatus(IJavaModelStatusConstants.DOM_EXCEPTION));
		}
	}
	
	//see doInheritChange()
	private boolean doUndoChange() {
		if (fBase == null || fDerived == null) {
			System.out.println("no inherit opreation possible, one of the arguments is null");
			return false;
	 	}	 	
	 	/*look if there is a working copy open in the java editor
		 * this is made because of sync reasons:
		 * if there is a working copy, work on the working copy to 
		 * ensure the changes to appear in the editor as well.
		 */
		try {
			editorInput = new FileEditorInput((IFile)fBase.getUnderlyingResource());
		}
		catch (JavaModelException jme) {
			jme.printStackTrace();
		}
    	ICompilationUnit workingCopy= JavaUI.getWorkingCopyManager().getWorkingCopy(editorInput);
    	if (workingCopy != null) {
    		/* this means there is a editor open showing the file
    		 * which is to be manipulated. Change the working copy,
    		 * not the original file (compilation unit):
    		 */
    		//do the change on the WORKINGCOPY:
    		workingCopy = undoChangeBase(workingCopy);
    		try {
    			commitChanges(new NullProgressMonitor(), true);	 
    		}
    		catch(CoreException ce) {
    			ce.printStackTrace();
    		}   		
    		return true;
    	}
    	else {
    		/* on editor is open, change the original
    		 * file (compilation unit), don't care about
    		 * any working copies:
    		 */
    		 fBase = undoChangeBase(fBase);
    		 try {
    		 	fBase.save(null, false);
    		 }
    		 catch(JavaModelException jme) {
    		 	jme.printStackTrace();
    		 }    	
    		 return true;	 
    	}
    	
	}
	
	/* (non java doc)
	 * this method sets the original contents (without the inheritence dependency) 
	 * to the underlying resource, the .java-file (compilation unit)
	 */
	private ICompilationUnit undoChangeBase(ICompilationUnit base) {
		if (fOriginalContentsBase == null) {
			System.out.println("Nothing to undo!");
			return null;
		}
		try {
			base.getBuffer().setContents(fOriginalContentsBase);
			if (base.isWorkingCopy()) {
	    		base.reconcile();
   			}
			return base;
		}
		catch(JavaModelException jme) {
			jme.printStackTrace();
			return null;
		}
	}
	
	/** getter method for the modified language element, the base class.
	 * @return the base class
	 */
	public ICompilationUnit getBase() {
		return fBase;
	}
	/** getter method for the superclass, the derived class.
	 * @return the derived class.
	 */
	public ICompilationUnit getDerived() {
		return fDerived;
	}
}
