package compositeRefactoring.refactoring;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.corext.refactoring.Checks;
import org.eclipse.jdt.internal.corext.refactoring.base.IChange;
import org.eclipse.jdt.internal.corext.refactoring.base.Refactoring;
import org.eclipse.jdt.internal.corext.refactoring.base.RefactoringStatus;
import org.eclipse.jdt.internal.corext.refactoring.changes.CreateCompilationUnitChange;
import org.eclipse.jdt.internal.corext.codemanipulation.CodeGenerationSettings;

import compositeRefactoring.changes.CreateAbstractClassChange;

/** This class is the representation of a refactoring performing an create abstract class transformation.
 * The class which will be created looks something like 
 * <code>public abstract class 'classname' {}</code>. It is also formatted.
 */

public class CreateAbstractClassRefactoring extends Refactoring {
	
	private IChange fChange;
	private String fCUName;
	private IPackageFragment fPackage;	
	
	/** creates a CreateAbstracClassRefactoring.
	 * @param pack the package in which the class (that is the compilation unit) is to be created.
	 * @param name the name of the new abstract class.
	 */
	public CreateAbstractClassRefactoring(IPackageFragment pack, String name) {
		fPackage = pack;
		fCUName = name;
		//get all superclasses.
		//make CodeGenerationSettings.
	}
	
	/** checks if there is a package in which the class can be created.
	 * @param pm the progress monitor which tracks the check.
	 * @return the status of the check.
	 */
	public RefactoringStatus checkActivation(IProgressMonitor pm) throws JavaModelException {
		return Checks.checkAvailability(fPackage);
	}
	/** checks if there is already a class in the package with this class's name.
	 * @param pm the progress monitor which tracks the check.
	 * @return the status of the check.
	 */
	public RefactoringStatus checkInput(IProgressMonitor pm) throws JavaModelException {
		/* can also be done differentely:
		 * the class compositeRefactoring.codemanipulation.CreateCompilationUnit
		 * already has a verify method which checks if the class already exists in this
		 * package. However, this class is just owned by the class 
		 * compositeRefactoring.changes.CreateAbstractClassChange. If a delegation is preferale, 
		 * have a look at the Inherit class set for reference.
		 */
		 		
		RefactoringStatus status = new RefactoringStatus();
		status.merge(Checks.checkCompilationUnitName(fCUName + ".java"));
		//check if fCUName already exists in fPackage
		ICompilationUnit[] packageUnits = fPackage.getCompilationUnits();
		for (int i = 0; i < packageUnits.length; i++) {
			if (Checks.isAlreadyNamed(packageUnits[i], (fCUName + ".java"))) {
				status.addFatalError("there is a compilation unit with name " + fCUName + " already in this package");
				return status;
			}
		}
		return status;
	}
	
	/** creates the change object for this refactoring. The change object is the actual transformation
	 * of code.
	 * @param pm the progress monitor tracking the change.
	 * @return the change object.
	 */
	public IChange createChange(IProgressMonitor pm) {
		fChange = new CreateAbstractClassChange(fPackage, fCUName);
		return fChange;
	}
	/** gets the name of the refactoring.
	 * @return the name of the refactoring.
	 */
	public String getName() {
		return "CreateAbstractClassRefactoring";
	}
	/** gets the new created compilation unit (the .java-file). It can be obtained for passing it to other refactorings.
	 * @return the new created compilation unit.
	 */
	public ICompilationUnit getNewCompilationUnit() {
		return ((CreateAbstractClassChange)fChange).getNewCompilationUnit();
	}
}
