package compositeRefactoring.refactoring;

import java.util.StringTokenizer;
import java.util.Vector;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.core.IJavaModelStatusConstants;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.Signature;
import org.eclipse.jdt.internal.core.JavaModelStatus;
import org.eclipse.jdt.internal.corext.refactoring.Checks;
import org.eclipse.jdt.internal.corext.refactoring.base.IChange;
import org.eclipse.jdt.internal.corext.refactoring.base.Refactoring;
import org.eclipse.jdt.internal.corext.refactoring.base.RefactoringStatus;

import compositeRefactoring.changes.CreateAbstractMethodChange;

/** This class is the representation of a refactoring performing create abstract method transformation.
 * A method will be created that looks something like
 * <code>public abstract 'returnType' 'methodName'('parameters');</code>
 */

public class CreateAbstractMethodRefactoring extends Refactoring {
	private IType fModifiedLanguageElement;
	private String fMethodName;
	private String fReturnType;
	private String[] fParameterTypes, fParameterNames, fParameters;
	private IChange fChange;
	
	/** creates a CreateAbstractMethodRefactoring object.
	 * @param parentType the method's parent type
	 * @param name the name of the new method
	 * @param returnType the return type of the new method
	 * @param parameters the parameters for the new method. They are suppposed to look something like
	 * {"Object o", "int i", "String s"}.
	 */
	public CreateAbstractMethodRefactoring(IType parentType, String name, String returnType, String[] parameters) {
		fModifiedLanguageElement = parentType;
		fMethodName = name;
		fReturnType = returnType;
		fParameters = parameters;
		try {
			dissolveParameters();
		}
		catch(JavaModelException jme) {
			jme.printStackTrace();
		}
	}
	/* (non java doc)
	 * transforms the parameter String array to one parameterType and one
	 * parameterName array
	 */
	private void dissolveParameters() throws JavaModelException {
		fParameterTypes = new String[fParameters.length];
		fParameterNames = new String[fParameters.length];
		for (int i = 0; i < fParameters.length; i++) {
			StringTokenizer st = new StringTokenizer(fParameters[i]);
			//must have two tokens!
			if (st.countTokens() != 2) {
				throw new JavaModelException(new JavaModelStatus(IJavaModelStatusConstants.NULL_STRING, "wrong parameter input"));
			}				
			fParameterTypes[i] = st.nextToken();
			fParameterNames[i] = st.nextToken();		
			
		}
		// for getting the signature of the parameterTypes. Not necessary yet.
		/*for (int i = 0; i < fParameterTypes.length; i++) {
			fParameterTypes[i] = Signature.createTypeSignature(fParameterTypes[i], false);
		}*/
	}
	
	/** checks if there is a modifyable IType in which 
	 * the new method can be created.
	 * @param pm the progress monitor which tracks the check.
	 * @return the status of the check.
	 */
	public RefactoringStatus checkActivation(IProgressMonitor pm) throws JavaModelException {
		RefactoringStatus status = new RefactoringStatus();
		//check if parentType exists and is modifyable
		Checks.checkAvailability(fModifiedLanguageElement);
		return status;
	}
	
	/** checks if the method already exists and if the type of parameters and return type is valid.
	 * @param pm the progress monitor which tracks the check.
	 * @return the status of the check.
	 */
	public RefactoringStatus checkInput(IProgressMonitor pm) throws JavaModelException {
		RefactoringStatus status = new RefactoringStatus();
		/* can also be done differentely:
		 * the class compositeRefactoring.codemanipulation.CreateAbstractMethod
		 * already has a verify method which checks if the method already exists in this
		 * package. However, this class is just owned by the class 
		 * compositeRefactoring.changes.CreateAbstractMethodChange. If a delegation is preferale, 
		 * have a look at the Inherit class set for reference.
		 */
		 
		/*check if method with this signature already exists in this type
		 * check if returnType is valid
		 * check if parameterTypes are valid
		 */
		 status.merge(checkSignatureExists());
		 status.merge(returnTypeValidation());
		 status.merge(parameterValidation());
		 return status;
	}

	/* (non java doc)
	 * checks if the signature of this method exists in the IType
	 */
	private RefactoringStatus checkSignatureExists() throws JavaModelException {
		RefactoringStatus status = new RefactoringStatus();
		Object[] methods = findMethodsByName(fMethodName, fModifiedLanguageElement);
		 for (int i = 0; i < methods.length; i++) {
		 	IMethod current = (IMethod) methods[i];
		 	if (current.getReturnType().equals(fReturnType)) {
		 		if (current.getNumberOfParameters() == fParameterTypes.length) {
		 			if (Checks.compareParamTypes(fParameterTypes, current.getParameterTypes())) {
		 				status.addError("Method already exists.");
		 				return status;
		 			}
		 		}
		 	}
		 }
		return status;
	}
	
	/* (non java doc) These two methods are not yet implemented.
	 * returnTypeValidation should check if the return Type is
	 * valid (after the java convention). There is no 
	 * method for checking types in Checks.java, only for checking
	 * type names. 
	 * parameterValidation should check if the parameterNames 
	 * are all valid (e.g. if they begin with a lower case letter)
	 * and if the  parameters' types are well defined.
	 */
	private RefactoringStatus returnTypeValidation() {
		return new RefactoringStatus();
	}
	private RefactoringStatus parameterValidation() {
		return new RefactoringStatus();
	}
	
	private Object[] findMethodsByName(String name, IType parent) {
		IMethod[] methods = null;
		try {
			methods = parent.getMethods();
		}
		catch(JavaModelException jme) {
			jme.printStackTrace();
		}
		Vector v = new Vector();
		for (int i = 0; i< methods.length; i++) {
			if (methods[i].getElementName().equals(name)) {
				v.add(methods[i]);
			}
		}
		return v.toArray();
	}
	
	/** creates the change object for this refactoring. The change object is the actual transformation
	 * of code.
	 * @param pm the progress monitor tracking the change.
	 * @return the change object.
	 */
	public IChange createChange(IProgressMonitor pm)
		throws JavaModelException {
		fChange = new CreateAbstractMethodChange(fModifiedLanguageElement,
													fMethodName, fReturnType, fParameters);
		return fChange;
	}
	
	/** gets the name of the refactoring.
	 * @return the name of the refactoring.
	 */
	public String getName() {
		return "CreateAbstractMethodRefactoring";
	}

}
