package compositeRefactoring.refactoring;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.corext.refactoring.Checks;
import org.eclipse.jdt.internal.corext.refactoring.base.IChange;
import org.eclipse.jdt.internal.corext.refactoring.base.IRefactoring;
import org.eclipse.jdt.internal.corext.refactoring.base.Refactoring;
import org.eclipse.jdt.internal.corext.refactoring.base.RefactoringStatus;

import compositeRefactoring.changes.InheritChange;
import compositeRefactoring.codemanipulation.Inherit;

/** This class is the representation of a refactoring performing an inheritence transformation.
 * Base inherits from Derived.
 * Base's superclass is changed to Derived unless one of the following enabling conditions are not 
 * fulfilled:<ul>
 * <li> Base is not superclass of Derived. Otherwise, the InheritChange would cause a cyclic inheritance hierarchy.
 * <li> Base has no superclass yet.
 * <li>Derived is not equal to Base
 * </ul>    
 */
public class InheritRefactoring extends Refactoring implements IRefactoring {
	
	private IChange fChange;
	private ICompilationUnit fBase, fDerived;
	private Inherit fInherit;
	
	/** creates the InheritRefactoring object.
	 * @param base the subclass of the inheritence dependency
	 * @param derived the superclass of the inheritence dependency
	 */
	public InheritRefactoring(ICompilationUnit base, ICompilationUnit derived) {
		fBase = base;
		fDerived = derived;
	}
	
	/** checks if the refactoring can be activated, e.g. if the vompilation units
	 * of base and derived exist, whether they are modifyable
	 * @param pm a progress monitor tracking the check.
	 * @return the status of the check
	 */
	public RefactoringStatus checkActivation(IProgressMonitor pm) throws JavaModelException {
		RefactoringStatus status = new RefactoringStatus();
		//update fBase and fDerived:
		fBase.makeConsistent(new NullProgressMonitor());
		fDerived.makeConsistent(new NullProgressMonitor());
		//check if Cus exist
		status.merge(Checks.checkAvailability(fBase));
		status.merge(Checks.checkAvailability(fDerived));
		return status;
	}
	
	/** checks the enabling conditions of the inherit refactoring. The conditions are<ul>
	 * <li> base must not be superclass of derived (cyclic inheritence dependency)
	 * <li> base and derived must not be the same
	 * <li> all abstract methods of derived must be implemented in base</ul>
	 */
	public RefactoringStatus checkInput(IProgressMonitor pm) throws JavaModelException {
		RefactoringStatus status = new RefactoringStatus();
		//update fBase and fDerived: (in case some former refactoring has changed them)
		fBase.makeConsistent(new NullProgressMonitor());
		fDerived.makeConsistent(new NullProgressMonitor());
		fInherit = new Inherit(fBase, fDerived);
		//check the four conditions from Tokuda...only two are checkable:
		status.merge(fInherit.checkCycle());
		status.merge(fInherit.checkBaseSuperclass());
		//still to check if base is abstract, too! Then we dont have to implement all abstract methods of derived!
		status.merge(fInherit.checkAbstractMethods());		
		return status;
	}
	
	/** creates the change object for this refactoring. The change object is the actual transformation
	 * of code.
	 * @param pm the progress monitor tracking the change.
	 * @return the change object.
	 */
	public IChange createChange(IProgressMonitor pm)
		throws JavaModelException {
		fChange = new InheritChange(fBase, fDerived, fInherit);
		return fChange;
	}
	
	/** gets the name of the refactoring.
	 * @return the name of the refactoring.
	 */
	public String getName() {
		return "InheritRefactoring";
	}

}
