package compositeRefactoring.refactoringComposite;

import java.util.Iterator;
import java.util.Stack;
import java.util.Vector;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.corext.refactoring.CompositeChange;
import org.eclipse.jdt.internal.corext.refactoring.base.ChangeContext;
import org.eclipse.jdt.internal.corext.refactoring.base.IChange;
import org.eclipse.jdt.internal.corext.refactoring.base.IRefactoring;
import org.eclipse.jdt.internal.corext.refactoring.base.RefactoringStatus;
import org.eclipse.jdt.internal.ui.refactoring.changes.ChangeExceptionHandler;

/** This class is an implementation of RefactoringComposite with an <b>Iterator Pattern</b>.
 */
public class RefactoringComposite_IteratorImpl extends RefactoringComposite implements IRefactoring {
	
	//Iterator pattern:
	private class ComponentIterator implements Iterator {
		private Vector list;
		private int pointer;
		
		ComponentIterator(RefactoringComponent[] comps) {
			pointer = 0;
			list = new Vector(comps.length);
			for (int i = 0; i < comps.length; i++) {
				list.add(comps[i]);
			}
		}
		public boolean hasNext() {
			return (list.size() > pointer);
		}
		public Object next() {
			Object o = list.elementAt(pointer);
			pointer++;
			return o;
		}
		public void remove() {
			//do nothing.
		}				
	}
	//Stack for undo:
	private class ComponentStack implements Iterator {
		private Stack list;
		
		ComponentStack(RefactoringComponent[] comps) {
			list = new Stack();
			for (int i = 0; i < comps.length; i++) {
				list.push(comps[i]);
			}
		}
		public boolean hasNext() {
			return !(list.empty());
		}
		public Object next() {
			return list.pop();
		}
		public void remove() {
			//already done in next.
		}
	}
	
	private RefactoringComponent[] refactoringParts;
	
	/** creates a new RefactoringComposite_IteratorImpl object with a default name.
	 * @param comps the RefactoringComponent s of this composite.
	 */
	public RefactoringComposite_IteratorImpl(RefactoringComponent[] comps) {
		this(DEFAULTNAME, comps);
	}	
	/** creates a new RefactoringComposite_IteratorImpl object.
	 * @param name the name of the RefactoringComposite.
	 * @param comps the RefactoringComponent s of this composite.
	 */
	public RefactoringComposite_IteratorImpl(String name, RefactoringComponent[] comps) {
		super(name);
		refactoringParts = comps;
	}		
	/** creates an instance of an <code>Iterator</code> with which the composite's components 
	 * can be visited.
	 * @return the <code>Iterator</code> for the composite's components.
	 */
	public Iterator iterator() {
		return new ComponentIterator(refactoringParts);
	}
	/** creates an instance of a <code>Stack</code> with which the composite's components 
	 * can be visited from top to bottom.
	 * @return the <code>Stack</code> for the composite's components.
	 */
	public Iterator stack() {
		return new ComponentStack(refactoringParts);
	}
				
	/** this method performs the composite refactoring. 
	* @return the status of the performed refactoring
	*/
	public RefactoringStatus refactor() {
		RefactoringStatus status = new RefactoringStatus();
	 	RefactoringComponent node;
	 	for (Iterator iter = iterator(); iter.hasNext();) {
	 		node = (RefactoringComponent)iter.next();
	 		//DEBUG:
	 		System.out.println("Doing refactoring " + node.getName());
	 		status.merge(node.refactor());
	 		if (status.getSeverity() == RefactoringStatus.INFO) {
	 			/* status is either WARNING, ERROR or FATALERROR, operation will be cancelled */
	 			System.out.println("can't do Composite Refactoring.\nUndoing changes made...");
	 			undo();
	 			return status;
	 		}	 		
	 	}
	 	return status;
	}	
	/** this method undos the composite refactoring when it was performed.
	 * @return the status of the Undo
	 */
	public RefactoringStatus undo() {
		RefactoringStatus status = new RefactoringStatus();
		RefactoringComponent node;
	 	//undo must be performed from top to bottom:	 	
	 	for(Iterator iter = stack(); iter.hasNext() ;) {
	 		node = (RefactoringComponent)iter.next();
	 		//DEBUG: 
	 		System.out.println("Doing undo for " + node.getName());
	 		status.merge(node.undo());
	 	}
	 	return status;		 	
	 }
	/** checks the Preconditions of the refactoring. This method calls an <code>Iterator</code>
	 * which calls the checkPrecondition methods of the RefactoringComposite children.
	 * @param pm an IProgressMonitor to monitor the method's progress
	 * @return the status of the checks.
	 */
	public RefactoringStatus checkPreconditions(IProgressMonitor pm) throws JavaModelException {
		RefactoringComponent next;
		RefactoringStatus status = new RefactoringStatus();
		for(Iterator iter = iterator(); iter.hasNext();) {
			next = (RefactoringComponent)iter.next();
			//DEBUG:
			System.out.println("checking preconditions for " + next.getName());
			status.merge(next.checkPreconditions(pm));
		}
		return status;
	}
	/** creates the change which is to be executed when performing the refactoring. This change 
	 * will <b>not</b> be performed, the method exists just for extensibility and information! 
	 * It is never called by a client currently.
	 * @param pm an IProgressMonitor to monitor the method's progress
	 * @return the actual change
	 */
	public IChange createChange(IProgressMonitor pm) throws JavaModelException {
		CompositeChange newChange = new CompositeChange("RefactoringCompositeChange");
		for(Iterator iter = iterator(); iter.hasNext();) {
			newChange.add(((RefactoringComponent)iter.next()).createChange(pm));
		}
		return newChange;
	}
	
}
