package compositeRefactoring.refactoringComposite;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.corext.refactoring.CompositeChange;
import org.eclipse.jdt.internal.corext.refactoring.base.ChangeContext;
import org.eclipse.jdt.internal.corext.refactoring.base.IChange;
import org.eclipse.jdt.internal.corext.refactoring.base.IRefactoring;
import org.eclipse.jdt.internal.corext.refactoring.base.IUndoManager;
import org.eclipse.jdt.internal.corext.refactoring.base.Refactoring;
import org.eclipse.jdt.internal.corext.refactoring.base.RefactoringStatus;
import org.eclipse.jdt.internal.ui.refactoring.changes.ChangeExceptionHandler;

/** This class is an implementation of RefactoringComposite with a List (java.util.List).
 */
public class RefactoringComposite_ListImpl extends RefactoringComposite implements IRefactoring {
	
	/* (non java-doc) 
	 * use Iterator pattern for List:
	 * iter=refactoringParts.iterator()::Iterator
	 * iter.hasNext(), iter.next(), iter.remove()
	 */
	
	/* This is not the Composite pattern yet...
	 * still to implement.
	 */
	 
	private List fRefactoringParts;
	
	/** creates a new RefactoringComposite_ListImpl object with a default name.
	 */
	public RefactoringComposite_ListImpl() {
		this(DEFAULTNAME);
	}
	/** creates a new RefactoringComposite_ListImpl object.
	 * @param name the name of the CompositeRefactoring.
	 */
	public RefactoringComposite_ListImpl(String name) {
		this(name, new ArrayList(5));
	}
	/** creates a new RefactoringComposite_ListImpl object and adds the RefactoringComponents to the list.
	 * @param name the name of the CompositeRefactoring.
	 * @param nodes the array of RefactoringComponents of this CompositeRefactoring
	 */
	public RefactoringComposite_ListImpl(String name, RefactoringComponent[] nodes) {
		this(name, new ArrayList(nodes.length));
		addAll(nodes);
	}
	/** creates a new RefactoringComposite_ListImpl object and adds the RefactoringComponents to the list.
	 * @param name the name of the CompositeRefactoring.
	 * @param refParts a <code>List</code> of the RefactoringComponent s.
	 */
	public RefactoringComposite_ListImpl(String name, List refParts) {
		super(name);
		fRefactoringParts = refParts;
	}
	/** adds all elements of the array to the Compositelist.
	 * @param nodes the array of RefactoringComponent s to be added.
	 */
	public void addAll(RefactoringComponent[] nodes) {
		for (int i = 0; i < nodes.length; i++) {
			add(nodes[i]);
		}
	}	
	/** adds one RefactoringComponent to the CompositeList.
	 * @param node the RefactoringComponent to be added.
	 */
	public void add(RefactoringComponent node) {
		fRefactoringParts.add(node);
	}			

	/** checks the Preconditions of the refactoring. This method calls an <code>Iterator</code>
	 * which calls the checkPrecondition methods of the RefactoringComposite children.
	 * @param pm an IProgressMonitor to monitor the method's progress
	 * @return the status of the checks.
	 */
	public RefactoringStatus checkPreconditions(IProgressMonitor pm) throws JavaModelException {
		RefactoringStatus status = new RefactoringStatus();
		for(Iterator iter = fRefactoringParts.iterator(); iter.hasNext();) {
			status.merge(((RefactoringNode)iter.next()).checkPreconditions(pm));
		}
		return status;
	}
	/** creates the change which is to be executed when performing the refactoring. This change 
	 * will <b>not</b> be performed, the method exists just for extensibility and information! 
	 * It is never called by a client currently.
	 * @param pm an IProgressMonitor to monitor the method's progress
	 * @return the actual change
	 */
	public IChange createChange(IProgressMonitor pm) throws JavaModelException {
		CompositeChange newChange = new CompositeChange("RefactoringCompositeChange");
		for(Iterator iter = fRefactoringParts.iterator(); iter.hasNext();) {
			newChange.add(((RefactoringNode)iter.next()).createChange(pm));
		}
		return newChange;
	}
	
	
	 /** this method performs the composite refactoring. 
	  * @return the status of the performed refactoring
	  */
	 /* (non-java-doc) there's no need to append the UndoManagers
	  * to the List since they will be appended in the RefactoringNode(s)!
	  */
	 public RefactoringStatus refactor() {
	 	RefactoringStatus status = new RefactoringStatus();
	 	RefactoringNode node;
	 	for (Iterator iter = fRefactoringParts.iterator(); iter.hasNext();) {
	 		node = (RefactoringNode)iter.next();
	 		//DEBUG:
	 		System.out.println("Doing refactoring " + node.getName());
	 		status.merge(node.refactor());
	 		if (status.getSeverity() > RefactoringStatus.INFO) {
	 			/* status is either WARNING, ERROR or FATALERROR, operation will be cancelled */
	 			System.out.println("can't do Composite Refactoring.\nUndoing changes made...");
	 			undo();
	 			return status;
	 		}	 		
	 	}
	 	return status;	 	 
	 }
	 /** undos the refactoring's change. The changes need to be undone from the last performed
	  * refactoring to the first as in a stack.
	  * @return the status of the undo operation.
	  */
	 public RefactoringStatus undo() {
	 	RefactoringStatus status = new RefactoringStatus();
	 	RefactoringNode node;
	 	for(int i = fRefactoringParts.size(); i >= 0; i--) {
	 		node = (RefactoringNode)fRefactoringParts.get(i);
	 		//DEBUG:
	 		System.out.println("Doing undo for " + node.getName());
	 		status.merge(node.undo());
	 	}
	 	return status;
	 }
	 
}
