package compositeRefactoring.refactoringComposite;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.corext.refactoring.base.ChangeContext;
import org.eclipse.jdt.internal.corext.refactoring.base.IChange;
import org.eclipse.jdt.internal.corext.refactoring.base.IRefactoring;
import org.eclipse.jdt.internal.corext.refactoring.base.IUndoManager;
import org.eclipse.jdt.internal.corext.refactoring.base.Refactoring;
import org.eclipse.jdt.internal.corext.refactoring.base.RefactoringStatus;
import org.eclipse.jdt.internal.ui.refactoring.CreateChangeOperation;
import org.eclipse.jdt.internal.ui.refactoring.PerformChangeOperation;
import org.eclipse.jdt.internal.ui.refactoring.changes.ChangeExceptionHandler;

/** A RefactoringNode is a representation of a single refactoring. This is a wrapper class
 * for Types of the class Refactoring. It implements the interface IRefactoring and, thus, 
 * has all methods on which Refactorings can operate.
 * Additionally, there are the inherited methods refactor() and undo() for executing and undoing the 
 * Refactoring without taking care of further delegations like IChange or core classes.
 */
public class RefactoringNode extends RefactoringComponent implements IRefactoring {
	
	//reference to the actual refactoring (from org.eclipse.jdt.internal.corext.refactoring...*)
	private Refactoring fRefactoring;
	private int fCheckedPassedSeverity;
	
	/** creates a new RefactoringNode component
	 * @param ref the refactoring object with which the transforamtion can be made.
	 */
	public RefactoringNode(Refactoring ref) {
		super(ref.getName());
		fRefactoring = ref;
		fCheckedPassedSeverity = RefactoringStatus.WARNING;
	}
	
	/** checks the Preconditions of the refactoring. 
	 * After performing some initial checks, the method delegates to the 
	 * actual refactoring object surrounded by this class.
	 * @param pm an IProgressMonitor to monitor the method's progress
	 * @return the status of the checks.
	 */
	public RefactoringStatus checkPreconditions(IProgressMonitor pm) throws JavaModelException {
		return fRefactoring.checkPreconditions(pm);
	}
	
	/**creates the change which is to be executed when performing the refactoring.
	 * The method delegates to the actual refactoring object surrounded by this class.
	 * It is not used in the current implementation - just for extensibility.
	 * @param pm an IProgressMonitor to monitor the method's progress
	 * @return the actual change
	 */
	public IChange createChange(IProgressMonitor pm) throws JavaModelException {
		return fRefactoring.createChange(pm);
	}
	
	/*public IUndoManager getUndoManager() {
		return fRefactoring.getUndoManager();
	}*/
	
	 /** this method performs the single refactoring. 
	  * @return the status of the performed refactoring
	  */
	 public RefactoringStatus refactor() {
	 	RefactoringStatus status = null;
	 	//check the preconditions:
	 	try {
	 		status = fRefactoring.checkPreconditions(new NullProgressMonitor());
	 	}
	 	catch(JavaModelException jme) {
	 		jme.printStackTrace();
	 	}
	 	if (status.getSeverity() > fCheckedPassedSeverity) {
	 		System.out.println("\tRefactoring " + fRefactoring.getName() + " can NOT be done with current severity");
	 		System.out.println("\tStatus: " + status.toString());
	 		//dont do refactoring!
	 		return status;
	 	}
	 	
	 	//perform refactoring	 	
	 	if (!doChange()) {
		 	status.addFatalError("REFACTORINGNODE: cannot perform refactoring!");
	 	}
	 	return status;
	 }
	 private boolean doChange() {
		//creating the change operation:
		//CHECK_NONE doesn't check preconditions since it was already check in refactor()
		CreateChangeOperation op = new CreateChangeOperation(fRefactoring, CreateChangeOperation.CHECK_NONE);
		op.setCheckPassedSeverity(fCheckedPassedSeverity);
		//performing the change...
		PerformChangeOperation pop = new PerformChangeOperation(op);
		pop.setCheckPassedSeverity(fCheckedPassedSeverity);
		boolean success = performFinish(pop);
		if (success) {
			//System.out.println("Refactoring done.\n");
			return true;
		}
		else {
			System.out.println("Refactoring not successful\n");	
			return false;
		}
	}	
	/* copied from org.eclipse.jdt.internal.ui.refactoring.RefactoringWizard.java
	 * modifications have been made. no exceptions will be handled.
	 * performFinish is performing the change and creating the undo information 
	 * (there are ExceptionHandlers missing...)
	 */	
	private boolean performFinish(PerformChangeOperation op) {
		IUndoManager fUndoManager = fRefactoring.getUndoManager();
		ChangeContext context= new ChangeContext(new ChangeExceptionHandler());
		boolean success= false;
		
		op.setChangeContext(context);
		fUndoManager.aboutToPerformRefactoring();
		try {
			op.run(new NullProgressMonitor());
		}
		catch(Exception e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
		finally {				
			if (op.changeExecuted()) {
				if (! op.getChange().isUndoable()){
					System.out.println("Change is NOT undoable\n");
					success= false;
				} else {
					fUndoManager.addUndo(fRefactoring.getName(), op.getChange().getUndoChange());
					success= true;
				}	
			}
			else {
				System.out.println("no change executed...\n");
				return false;
			}		
			context.clearPerformedChanges();
			fUndoManager.refactoringPerformed(success);	
			//System.out.println("Success?: " + success);
			return true;
		}
	}
	 
	 /** this method undos the single refactoring when it was performed.
	  * @return the status of the Undo
	  */
	 public RefactoringStatus undo() {
	 	IUndoManager myUndo = fRefactoring.getUndoManager();
	 	RefactoringStatus status = null;
	 	try {
			status = myUndo.performUndo(new ChangeContext(new ChangeExceptionHandler()), new NullProgressMonitor());
			//myUndo.flush();
		} catch (JavaModelException e) {
			System.out.println(e.getMessage());
		}
		return status;
	 }
	 //getter:
	 /** gets the Refactoring object of the RefactoringNode.
	  * @return the actual refactoring.
	  */
	 public Refactoring getRefactoring() {
	 	return fRefactoring;
	 }
	 /** sets the severity of the refactoring, that is the tolerance of errors which can be passed.
	  * possible severity levels are:<ul>
	  * <li>RefactoringStatus.ERROR: an error as created by a compiler
	  * <li>RefactoringStatus.INFO: neither error nor warning.
	  * <li>RefactoringStatus.WARNING: a compiler warning like <i>deprecated</i>
	  * <li>RefactoringStatus.OK: no warnings and errors found in this RefactoringStatus.
	  * <ul>@see RefactoringStatus
	  */
	public void setCheckedPassedSeverity(int i) {
		fCheckedPassedSeverity = i;
	}
}
